from argumentHelper import *

# Detect LibRapid features
features = []
try:
	with open("../../configuration.txt", "r") as file:
		for line in file:
			if len(line) == 0 or line.startswith("#"):
				continue
			
			print("FEATURE:", line.strip())
			try:
				args = line.split()
				features.append((args[0], args[2])) # Cut out the "="
			except:
				pass
except:
	pass

resStr = ""

types = [
	"mpz",
	"mpf",
	"mpq",
	"mpfr"
]

for type in types:
	t = type
	typename = "librapid::{}".format(t)
	constRef = "const librapid::{} &".format(t)
	ref = "librapid::{} &".format(t)
	int64_t = "int64_t"
	extent = "librapid::Extent"
	stdString = "std::string"
	stdStringConstRef = "const std::string &"

	# Class Definition
	resStr += "py::class_<{0}>(module, \"{1}\")\n".format(typename, t)

	# Constructors
	resStr += "\t.def(py::init<>())\n"
	resStr += "\t.def(py::init<int64_t>())\n"
	resStr += "\t.def(py::init<double>())\n"
	resStr += "\t.def(py::init<const std::string &>())\n"
	resStr += "\t.def(py::init<{}>())\n".format(constRef)

	functions = [
		Function("__add__", [Argument(constRef, "this_"), Argument(constRef, "other")],     "return this_ + other;"),
		Function("__sub__", [Argument(constRef, "this_"), Argument(constRef, "other")],     "return this_ - other;"),
		Function("__mul__", [Argument(constRef, "this_"), Argument(constRef, "other")],     "return this_ * other;"),
		Function("__truediv__", [Argument(constRef, "this_"), Argument(constRef, "other")], "return this_ / other;"),

		Function("__radd__", [Argument(constRef, "this_"), Argument(constRef, "other")],     "return other + this_;"),
		Function("__rsub__", [Argument(constRef, "this_"), Argument(constRef, "other")],     "return other - this_;"),
		Function("__rmul__", [Argument(constRef, "this_"), Argument(constRef, "other")],     "return other * this_;"),
		Function("__rtruediv__", [Argument(constRef, "this_"), Argument(constRef, "other")], "return other / this_;"),

		Function("__iadd__", [Argument(ref, "this_"), Argument(constRef, "other")],     "this_ += other; return this_;"),
		Function("__isub__", [Argument(ref, "this_"), Argument(constRef, "other")],     "this_ -= other; return this_;"),
		Function("__imul__", [Argument(ref, "this_"), Argument(constRef, "other")],     "this_ *= other; return this_;"),
		Function("__itruediv__", [Argument(ref, "this_"), Argument(constRef, "other")], "this_ /= other; return this_;"),
		
		Function("__lt__", [Argument(constRef, "this_"), Argument(constRef, "other")],  "return this_ < other;"),
		Function("__gt__", [Argument(constRef, "this_"), Argument(constRef, "other")],  "return this_ > other;"),
		Function("__lte__", [Argument(constRef, "this_"), Argument(constRef, "other")], "return this_ <= other;"),
		Function("__gte__", [Argument(constRef, "this_"), Argument(constRef, "other")], "return this_ >= other;"),

		Function("str", [Argument(constRef, "this_"), Argument("int64_t", "base", "10")], "return lrc::str(this_, {-1, base, false});"),
		Function("__str__", [Argument(constRef, "this_")], "return lrc::str(this_, {-1, 10, false});"),
		Function("__repr__", [Argument(constRef, "this_")], "return \"librapid::{}(\\\"\" + lrc::str(this_, {{-1, 10, false}}) + \"\\\")\";".format(t)),
	]

	if type != "mpfr":
		functions += [
			Function("__lshift__", [Argument(constRef, "this_"), Argument("int64_t", "other")], "return this_ << other;"),
			Function("__rshift__", [Argument(constRef, "this_"), Argument("int64_t", "other")], "return this_ >> other;"),
			Function("__ilshift__", [Argument(ref, "this_"), Argument("int64_t", "other")], "this_ <<= other; return this_;"),
			Function("__irshift__", [Argument(ref, "this_"), Argument("int64_t", "other")], "this_ >>= other; return this_;"),
		]

	for i in range(len(functions)):
		function = functions[i]
		if isinstance(function,Function):
			resStr += "\t" + function.gen()
		else:
			resStr += "\t" + function
		
		if i + 1 < len(functions):
			resStr += "\n"
		else:
			resStr += ";\n\n"

for type in types:
	functions = [
		Function("toMpz", [Argument(constRef, "this_")], "return librapid::toMpz(this_);"),
		Function("toMpf", [Argument(constRef, "this_")], "return librapid::toMpf(this_);"),
		Function("toMpq", [Argument(constRef, "this_")], "return librapid::toMpq(this_);"),
		Function("toMpfr", [Argument(constRef, "this_")], "return librapid::toMpfr(this_);"),
	]

	for function in functions:
		resStr += "module" + function.gen() + ";\n"

def write(path:str):
	with open(path, "w") as file:
		file.write("""
#include <librapid/librapid.hpp>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>
#include <pybind11/functional.h>
#include <functional>
#include <string>

// Just remove these. They're pointless
#ifdef min
#undef min
#endif

#ifdef max
#undef max
#endif

namespace lrc = librapid;
namespace py = pybind11;

""")

		file.write("void init_{}(py::module &module) {{\n".format(type))
		
		file.write(resStr)

		file.write("\n}")

if __name__ == "__main__":
	write("../autogen/mpfrInterface.cpp")

	print("\"${{CMAKE_CURRENT_SOURCE_DIR}}/src/librapid/python/autogen/mpfrInterface.cpp\"".format(type))
	print("void init_mpfr(py::module &);")
	print("init_mpfr(module);")
	print()



#
# Python script for running MPFR tests
#
# Run this from the build.vc11 directory

import sys, os, shutil, subprocess, filecmp

test = "dll"

test_dir = test + '_mpfr_tests\\'
lib_loc  = '..\\' + test + '\\'
lib_name = '\\mpfr.' + test

cw, f = os.path.split(__file__)
os.chdir(cw)

def write_f(ipath, opath):
  if os.path.exists(ipath) and not os.path.isdir(ipath):
    if os.path.exists(opath) and os.path.isfile(opath) and filecmp.cmp(ipath, opath):
      return
    dp, f = os.path.split(opath)
    try:
      os.mkdir(dp)
    except FileExistsError:
      pass
    shutil.copy2(ipath, opath)

def copy_dir(src, dst):
  os.makedirs(dst, exist_ok=True)
  for item in os.listdir(src):
    s = os.path.join(src, item)
    d = os.path.join(dst, item)
    if os.path.isdir(s):
      copy_dir(s, d)
    else:
      write_f(s, d)

# get a list of tests from the user
def get_input(n):
  li = []
  while True:
    s = "list the numbers of those you wish to test [1 - " + str(n) +"] ? "
    if sys.version[0] == '3':
      r = input(s)
    else:
      r = raw_input(s)
    tl = []
    if len(r) == 0:
      return [i for i in range(len(li))]
    if r.find(',') != -1:
      tl = [u for u in r.split(',')]
    elif r.find(' ') != -1:
      tl = [u for u in r.split(' ')]
    else:
      tl = [r]
    for it in tl:
      try:
        ind = int(it)
      except:
        print("you can only input the numbers of listed items")
        break
      if ind > 0 and ind <= n:
        li += [ind - 1]
      else:
        print(ind, "is not in list")
        break
    if len(li):
      return li

test_ext = ["Win32\\Release", "x64\\Release", "Win32\\Debug", "x64\\Debug" ]

# find libraries that have been built and list them in order of creation date
# for selection by the user

li = []
for x in test_ext:
  p = lib_loc + x + lib_name
  print(p)
  if os.path.exists(p):
    s = os.stat(p)
    li += [[s.st_mtime, x]]

if len(li) > 1:
  li.sort(key = lambda x : x[0])
  for i in range(len(li)):
    print("  ", i + 1, end='')
    if not i:
      print(": (old)", end='')
    elif i == len(li) - 1:
      print(": (new)", end='')
    else:
      print(":      ", end='')
    print(li[i][1])
  nl = get_input(len(li))
else:
  nl = [0]

# copy any required data into the test directory

shutil.copy( "..\\tests\\inp_str.dat", test_dir )
shutil.copy( "..\\tests\\tfpif_r1.dat", test_dir )
shutil.copy( "..\\tests\\tfpif_r2.dat", test_dir )
shutil.copy( "..\\tests\\tmul.dat", test_dir )
if os.path.exists("..\\tests\\data\\"):
  if not os.path.exists(test_dir + "\\data\\"):
    shutil.copytree("..\\tests\\data\\", test_dir + "\\data\\")
  else:
    copy_dir("..\\tests\\data\\", test_dir + "\\data\\")

# generate list of projects from *.vcproj files

prj_list = []
for p, d, f in os.walk(test_dir):
  for ff in f:
    x = os.path.splitext(ff)
    if x[1] == '.vcxproj' and x[0] != 'lib_tests':
      prj_list += [x[0]]
prj_list.sort()

# go through each library to be tested

for k in nl:
  tpos = test_dir + li[k][1] + "\\"
  print("Testing",  lib_name, "in", li[k][1], ":")

  if "dll" in lib_loc:
    # copy gmp.dll and mpfr.dll into the test directory
    gpos = "..\\..\\mpir\\dll\\" + li[k][1] + "\\mpir.dll"
    mpos = lib_loc + li[k][1] + lib_name
    shutil.copy(gpos, tpos)
    shutil.copy(mpos, tpos)

  # list the *.exe files in the test directory
  exe_list = []
  try:
    l = os.listdir(tpos)
  except:
    print("Tests have not been built for this configuration")
    os._exit(-1)

  for f in l:
    x = os.path.splitext(f)
    if x[1] == '.exe':
      exe_list += [x[0]]
  if len(exe_list) == 0:
    print("No executable test files for this configuration")
    os._exit(-1)

  # now test projects that have been built and the build failures
  build_fail = 0
  run_ok = 0
  run_fail = 0
  skipped = 0
  for i in prj_list:
    if i in exe_list:
      ef = test_dir + li[k][1] + '\\' + i + '.exe'
      prc = subprocess.Popen( ef, stdout = subprocess.PIPE, cwd = test_dir,
        stderr = subprocess.STDOUT, creationflags = 0x08000000 )
      output = prc.communicate()[0]
      if prc.returncode:
        if prc.returncode == 77:
          print(i, ': test skipped')
          skipped += 1
        else:
          print(i, ': ERROR (', prc.returncode, ' )')
          run_fail += 1
          if output:
            print('    ', output.decode(), end = '')
      else:
        print(i, ': success')
        run_ok += 1
    else:
      print("Build failure for {0}".format(i))
      build_fail += 1
  print(build_fail + run_ok + run_fail + skipped, "tests:")
  if build_fail > 0:
    print("\t{0} failed to build".format(build_fail))
  if run_ok > 0:
    print("\t{0} ran correctly".format(run_ok))
  if run_fail > 0:
    print("\t{0} failed".format(run_fail))
  if skipped > 0:
    print("\t{0} skipped".format(skipped))
if len(sys.argv) == 1:
  try:
    input(".. completed - press ENTER")
  except:
    pass
else:
  sys.exit(build_fail + run_fail)



def cast(args, scalarTypes, retType, allowed=None, extractor=None):
	if isinstance(args, retType):
		return retType(args)

	if allowed is not None and extractor is not None:
		if isinstance(args, allowed):
			return extractor(args)
	
	if isinstance(args, (list, tuple)):
		if len(args) == 0:
			return None

		# Check for a list type
		if allowed is not None:
			typeList = (list, tuple, retType, allowed)
		else:
			typeList = (list, tuple, retType)

		if isinstance(args[0], typeList):
			if len(args) > 1:
				return None
			return cast(args[0], scalarTypes, retType, allowed, extractor)

		# Cast elements in args to return type
		for val in args:
			if not isinstance(val, scalarTypes):
				return None
			return retType(args)
			


from argumentHelper import *

# Detect LibRapid features
features = []
try:
	with open("../../configuration.txt", "r") as file:
		for line in file:
			if len(line) == 0 or line.startswith("#"):
				continue
			
			print("FEATURE:", line.strip())
			try:
				args = line.split()
				features.append((args[0], args[2])) # Cut out the "="
			except:
				pass
except:
	pass

arrayTypes = [
	"ArrayBG",
	"ArrayCG",
	"ArrayF16G",
	"ArrayF32G",
	"ArrayF64G",
	"ArrayI16G",
	"ArrayI32G",
	"ArrayI64G",

	"ArrayB",
	"ArrayC",
	"ArrayF16",
	"ArrayF32",
	"ArrayF64",
	"ArrayI16",
	"ArrayI32",
	"ArrayI64",
	
	"ArrayMPZ",
	"ArrayMPQ",
	"ArrayMPFR",

	"ArrayCF32",
	"ArrayCF64",
	"ArrayCMPFR",
]

classStr = ""
moduleStr = ""

interfaceList = {}

for t in arrayTypes:
	typename = "librapid::{}".format(t)
	constRef = "const librapid::{} &".format(t)
	ref = "librapid::{} &".format(t)
	int64_t = "int64_t"
	scalar = "typename librapid::internal::traits<{}>::Scalar".format(typename)
	extent = "librapid::Extent"
	extentConstRef = "const librapid::Extent &"
	stdString = "std::string"
	stdStringConstRef = "const std::string &"

	# Class Definition
	classStr += "py::class_<{0}>(module, \"{1}\")\n".format(typename, t)

	# Constructors
	classStr += "\t.def(py::init<>())\n"
	classStr += "\t.def(py::init<librapid::Extent>())\n"
	classStr += "\t.def(py::init<{}>())\n".format(constRef)
	classStr += "\t.def(py::init<librapid::internal::traits<{}>::Scalar>())\n".format(typename)

	fCopy = [
		Function("copy", [Argument(constRef, "this_")], "return this_.copy();"),
	]

	fIndex = [
		Function("__getitem__", [Argument(constRef, "this_"), Argument(int64_t, "index")], "return this_[index];"),
		Function("__setitem__", [Argument(ref, "this_"), Argument(int64_t, "index"), Argument(scalar, "val")], "this_[index] = val;"),
		Function("__setitem__", [Argument(ref, "this_"), Argument(int64_t, "index"), Argument(scalar, "val")], "this_[index] = val;")
	]

	for i in range(1, 32):
		args = [Argument("int64_t", "index{}".format(ind)) for ind in range(i)]
		params = ", ".join(["index{}".format(ind) for ind in range(i)])
		fIndex.append(Function("__call__", [Argument(constRef, "this_")] + args, "return this_({}).get();".format(params)))
		fIndex.append(Function("get", [Argument(constRef, "this_")] + args, "return this_({}).get();".format(params)))
		fIndex.append(Function("set", [Argument(constRef, "this_"), Argument(scalar, "val")] + args, "this_({}) = val;".format(params)))

	fMove = [
		Function("move_CPU", [Argument(constRef, "this_")], "return this_.move<librapid::device::CPU>();"),
	]

	fMove.append("#if defined(LIBRAPID_HAS_CUDA)")
	fMove.append(Function("move_GPU", [Argument(constRef, "this_")], "return this_.move<librapid::device::GPU>();"))
	fMove.append("#endif // LIBRAPID_HAS_CUDA")

	fCast = []
	
	for t2 in arrayTypes:
		if t2[0] == "#":
			continue

		typename2 = "librapid::{}".format(t2)
		scalar2 = "typename librapid::internal::traits<{}>::Scalar".format(typename2)
		fCast += [
			Function("cast_{}".format(t2), [Argument(constRef, "this_")], "return this_.cast<{}>();".format(scalar2)),
			# Function("castMove_{}_CPU".format(t2), [Argument(constRef, "this_")], "return this_.castMove<{}, librapid::device::CPU>();".format(scalar2)),
		]

		# fCast.append("#if defined(LIBRAPID_HAS_CUDA)")
		# fCast.append(Function("castMove_{}_GPU".format(t2), [Argument(constRef, "this_")], "return this_.castMove<{}, librapid::device::GPU>();".format(scalar2)))
		# fCast.append("#endif // LIBRAPID_HAS_CUDA")

	if t not in ["ArrayB", "ArrayBG"]:
		fArithmetic = [
			Function("__add__", [Argument(constRef, "this_"), Argument(constRef, "other")], "return this_ + other;"),
			Function("__add__", [Argument(constRef, "this_"), Argument(scalar, "other")], "return this_ + other;"),

			Function("__sub__", [Argument(constRef, "this_"), Argument(constRef, "other")], "return this_ - other;"),
			Function("__sub__", [Argument(constRef, "this_"), Argument(scalar, "other")], "return this_ - other;"),

			Function("__mul__", [Argument(constRef, "this_"), Argument(constRef, "other")], "return this_ * other;"),
			Function("__mul__", [Argument(constRef, "this_"), Argument(scalar, "other")], "return this_ * other;"),

			Function("__div__", [Argument(constRef, "this_"), Argument(constRef, "other")], "return this_ / other;"),
			Function("__div__", [Argument(constRef, "this_"), Argument(scalar, "other")], "return this_ / other;"),
		]
	else:
		fArithmetic = []

	if t not in ["ArrayF16", "ArrayF16G", "ArrayF32", "ArrayF32G", "ArrayF64", "ArrayF64G", "ArrayMPFR", "ArrayMPQ", "ArrayCF32", "ArrayCF64", "ArrayCMPFR"]:
		fBitwise = [
			Function("__or__", [Argument(constRef, "this_"), Argument(constRef, "other")], "return this_ | other;"),
			Function("__and__", [Argument(constRef, "this_"), Argument(constRef, "other")], "return this_ & other;"),
			Function("__xor__", [Argument(constRef, "this_"), Argument(constRef, "other")], "return this_ ^ other;"),
		]
	else:
		fBitwise = []

	fUnary = []
	if t not in ["ArrayF16", "ArrayF16G", "ArrayF32", "ArrayF32G", "ArrayF64", "ArrayF64G", "ArrayMPFR", "ArrayMPQ", "ArrayCF32", "ArrayCF64", "ArrayCMPFR"]:
		fUnary += [Function("__invert__", [Argument(constRef, "this_")], "return ~this_;")]

	if not t.startswith("ArrayB"):
		fUnary += [Function("__neg__", [Argument(constRef, "this_")], "return -this_;")]

	fMatrix = [
		Function("transpose", [Argument(ref, "this_"), Argument(extentConstRef, "order", "{}")], "this_.transpose(order);"),
		Function("transposed", [Argument(constRef, "this_"), Argument(extentConstRef, "order", "{}")], "return this_.transposed(order);"),
		Function("dot", [Argument(constRef, "this_"), Argument(constRef, "other")], "return this_.dot(other);"),
	]

	fString = [
		Function("str", [Argument(constRef, "this_"),
						 Argument(stdStringConstRef, "format", "\"{}\""),
						 Argument(stdStringConstRef, "delim", "\" \""),
						 Argument(int64_t, "stripWidth", "-1"),
						 Argument(int64_t, "beforePoint", "-1"),
						 Argument(int64_t, "afterPoint", "-1"),
						 Argument(int64_t, "depth", "0")], "return this_.str(format, delim, stripWidth, beforePoint, afterPoint, depth);"),
		Function("__str__", [Argument(constRef, "this_")], "return this_.str();"),
		Function("__repr__", [Argument(constRef, "this_")], "return \"<{0}\\n\" + this_.str(\"{{}}\", \",\") + \"\\n>\";".format(typename))
	]
	
	fProperties = [
		Function("isScalar", [Argument(constRef, "this_")], "return this_.isScalar();"),
		Function("extent", [Argument(constRef, "this_")], "return this_.extent();"),
	]

	functions = fCopy + fIndex + fMove + fCast + fArithmetic + fBitwise + fUnary + fMatrix + fString + fProperties

	for i in range(len(functions)):
		function = functions[i]
		if isinstance(function,Function):
			classStr += "\t" + function.gen()
		else:
			classStr += "\t" + function
		
		if i + 1 < len(functions):
			classStr += "\n"
		else:
			classStr += ";\n"

	# Module-based functions

	if t not in ("ArrayB", "ArrayBG"):
		forceTmpFunc = [
			Function("add", [Argument(constRef, "lhs"), Argument(constRef, "rhs"), Argument(ref, "dst")], "librapid::add(lhs, rhs, dst);"),
			Function("sub", [Argument(constRef, "lhs"), Argument(constRef, "rhs"), Argument(ref, "dst")], "librapid::sub(lhs, rhs, dst);"),
			Function("mul", [Argument(constRef, "lhs"), Argument(constRef, "rhs"), Argument(ref, "dst")], "librapid::mul(lhs, rhs, dst);"),
			Function("div", [Argument(constRef, "lhs"), Argument(constRef, "rhs"), Argument(ref, "dst")], "librapid::div(lhs, rhs, dst);"),
		]
	else:
		forceTmpFunc = []

	moduleFunctions = forceTmpFunc

	for function in moduleFunctions:
		moduleStr += "module" + function.gen() + ";\n"

	interfaceList[t] = classStr + "\n\n" + moduleStr
	classStr = ""
	moduleStr = ""

# def write(path:str):
# 	with open(path, "w") as file:
# 		file.write(classStr)
# 		file.write("\n")
# 		file.write(moduleStr)

def write(path:str):
	for type, interface in interfaceList.items():
		with open(path + "/{}Interface.cpp".format(type), "w") as file:
			file.write("""
#include <librapid/librapid.hpp>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>
#include <pybind11/functional.h>
#include <functional>
#include <string>

// Just remove these. They're pointless
#ifdef min
#undef min
#endif

#ifdef max
#undef max
#endif

namespace lrc = librapid;
namespace py = pybind11;

""")

			file.write("void init_{}(py::module &module) {{\n".format(type))
			if type[-1] == "G":
				file.write("#if defined(LIBRAPID_HAS_CUDA)\n")
			file.write(interface)
			if type[-1] == "G":
				file.write("#endif\n")
			file.write("\n}")

if __name__ == "__main__":
	write("../autogen")

	for type, interface in interfaceList.items():
		print("\"${{CMAKE_CURRENT_SOURCE_DIR}}/src/librapid/python/autogen/{}Interface.cpp\"".format(type))

	print("\n")

	for type, interface in interfaceList.items():
		print("void init_{}(py::module &);".format(type))

	print("\n")

	for type, interface in interfaceList.items():
		print("init_{}(module);".format(type))



# -*- coding: utf-8 -*-
import os
import platform
import shutil
import sys
import site
import pathlib
from setuptools import find_packages
from packaging.version import LegacyVersion
from skbuild import setup
from skbuild.cmaker import get_cmake_version
from skbuild.exceptions import SKBuildError

LR_PYTHON_IMPL = platform.python_implementation()

# Copy OpenBLAS build if present in the root directory
if os.path.exists("openblas_install") and not os.path.exists(os.path.join("src", "librapid", "openblas_install")):
    shutil.copytree("openblas_install", os.path.join("src", "librapid", "openblas_install"))

# Remove _skbuild directory if it already exists. It can lead to issues
if os.path.exists("_skbuild"):
    shutil.rmtree("_skbuild")

# Remove the _librapid_python_cmake directory if it's present. This can cause more issues...
if os.path.exists("_librapid_python_cmake"):
    shutil.rmtree("_librapid_python_cmake")

# If the directory "src/librapid/blas" is empty and "src/librapid/openblas_install" is empty,
# run CMake to automatically detect BLAS before installing the Python library
if not os.path.exists(os.path.join("src", "librapid", "blas")) and not os.path.exists(os.path.join("src", "librapid", "openblas_install")):
    out = os.system("mkdir _librapid_python_cmake && cd _librapid_python_cmake && cmake ..")
    if out != 0:
        print("\nCMake failed to run correctly, so it is likely that BLAS will not be installed with LibRapid")

ROOT_DIR = os.path.dirname(os.path.abspath(__file__))

if not os.path.exists("src/librapid/python/autogen"):
    # Force-create the directory
    os.mkdir(os.path.join("src", "librapid", "python", "autogen"))

    # Run the autogen files
    genPath = os.path.join("src", "librapid", "python", "generators")
    vecInterfacePath = os.path.join("src", "librapid", "python", "autogen", "vecInterface.hpp")
    extentInterfacePath = os.path.join("src", "librapid", "python", "autogen", "extentInterface.hpp")
    arrayInterfacePath = os.path.join("src", "librapid", "python", "autogen", "arrayInterface.hpp")

    sys.path.append(genPath)
    import vecInterface
    import extentInterface
    import arrayInterface

    vecInterface.write(vecInterfacePath)
    extentInterface.write(extentInterfacePath)
    arrayInterface.write(arrayInterfacePath)

# Add CMake as a build requirement if cmake is not installed or is too low a version
setup_requires = []
install_requires = []

try:
    if LegacyVersion(get_cmake_version()) < LegacyVersion("3.10"):
        setup_requires.append('cmake')
        install_requires.append("cmake")
except SKBuildError:
    setup_requires.append('cmake')
    install_requires.append("cmake")

# ======= Uncomment this to install win32api as well =======
if platform.system() == "Windows" and LR_PYTHON_IMPL == "CPython":
    setup_requires.append('pywin32')
    install_requires.append("pywin32")

data_files = []
if platform.system() == "Windows":
    try:
        root, lib = site.getsitepackages()
        libdir = lib.replace(root, "")
        libdir = libdir.lstrip("\\")
        libdir = libdir.lstrip("/")
    except:
        from distutils.sysconfig import get_python_lib
        libdir = get_python_lib()
    
    dll_files = ["openblas.dll", "libopenblas.dll", "flang.dll", "flangrti.dll", "pgmath.dll", "libomp.dll"]
    
    for filename in dll_files:
        try:
            print("Attempting to open 'src/librapid/blas/{}".format(filename))
            with open(os.path.join("src", "librapid", "blas", filename), "r") as _:
                data_files.append(os.path.join("src", "librapid", "blas", filename))
        except:
            print("Failed to open 'src/librapid/blas/bin/{}'".format(filename))
            pass

    if data_files == []:
        for filename in dll_files:
            try:
                print("Attempting to open 'src/librapid/openblas_install/bin/{}'".format(filename))
                with open(os.path.join("src", "librapid", "openblas_install", "bin", filename), "r") as _:
                    data_files.append(os.path.join("src", "librapid", "openblas_install", "bin", filename))
            except:
                print("Failed to open 'src/librapid/openblas_install/bin/{}'".format(filename))
                pass

	# Log some information
    if data_files == []:
        print("Was not able to load datafiles")
        print("File information:")
        if os.path.exists("src"):
            print("./src")
            print(os.listdir("./src"))

            if (os.path.exists("src/librapid")):
                print("./src/librapid")
                print(os.listdir("./src/librapid"))

                if (os.path.exists("src/librapid/openblas_install")):
                    print("./src/librapid/openblas_install")
                    print(os.listdir("./src/librapid/openblas_install"))

                    if (os.path.exists("src/librapid/openblas_install/bin")):
                        print("./src/librapid/openblas_install/bin")
                        print(os.listdir("./src/librapid/openblas_install/bin"))

# Adjust the datafiles object
if data_files != []:
	data_files = [(os.path.join(".", libdir, "librapid"), data_files[:])]

pythonApi = list(pathlib.Path("src/librapid/python").rglob("*.[tT][xX][tT]"))
data_files += [("librapid", file) for file in pythonApi]

print("Operating System: {}".format(platform.system()))
print("Additional files being included: {}".format(data_files))
print("\n\n\n\n")

# Load the version number from VERSION.hpp
version_file = open(os.path.join("src", "librapid", "VERSION.hpp"), "r")
__version__ = version_file.readlines()[1].split()[2].replace("\"", "")
version_file.close()

# Locate and read the contents of README.md
with open(os.path.join(ROOT_DIR, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

if sys.maxsize > 2 ** 32 and platform.system() == "Windows":
    # print("Using 64bit Python")
    cmake_args = ["-DCMAKE_GENERATOR_PLATFORM=x64"]
else:
    # print("Using 32bit Python (or an OS other than Windows)")
    cmake_args = []

if os.environ.get("LIBRAPID_NO_ARCH"):
    cmake_args.append("-DLIBRAPID_NO_ARCH=on")

cmake_args.append("-DLIBRAPID_USE_MULTIPREC=on")

if os.environ.get("LIBRAPID_CUDA_WHEEL"):
    moduleName = "librapid_cuda_" + os.environ["LIBRAPID_CUDA_WHEEL"]
else:
    moduleName = "librapid"

# cmake_args.append("-DCMAKE_BUILD_TYPE=Release")
# cmake_args.append("-DCMAKE_BUILD_TYPE=DEBUG")

setup(
    name=moduleName,
    version=__version__,
    author="Toby Davis",
    author_email="pencilcaseman@gmail.com",
    url="https://github.com/Pencilcaseman/librapid",
    description="A fast math and neural network library for Python and C++",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=["librapid"] + ["librapid." + mod for mod in find_packages("src/librapid")],
    package_dir={"": "src"},
    cmake_args=cmake_args,
    cmake_install_dir="src/librapid",
    license="MIT License",
    keywords=["math", "neural network", "ndarray", "array", "matrix",
              "high-performance computing"],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
		"Programming Language :: Python :: 3.10",
    ],
    extras_require={"test": "pytest"},
    install_requires=install_requires,
    setup_requires=setup_requires,
	data_files=data_files,
	include_package_data=True,
    zip_safe=False
)
